--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}

petbm.MainView = {}
petbm.MainView.prototype = {}
petbm.MainView.metatable = {__index = petbm.MainView.prototype}

local L = petbm.Locale.GetInstance()
local log = petbm.Debug:new("MainView")

local function _RestoreWindow(self)
	local save = self.profile.windowPosition
	if (save) then
		if (save.point) then
			self.frame:ClearAllPoints()
			self.frame:SetPoint(save.point, UIParent, save.relativePoint, save.x, save.y)
		end
	end
end

local function _SaveWindowPosition(frame)
	log:Debug("SaveWindowPosition")
	local self = frame.obj
	if (not self.profile.windowPosition) then
		self.profile.windowPosition = {}
	end
	local save = self.profile.windowPosition
	local point, relativeTo, relativePoint, x, y = self.frame:GetPoint(1)
	save.point = point
	save.relativePoint = relativePoint
	save.x = math.floor(x + 0.5)
	save.y = math.floor(y + 0.5)
end

local function _InitFrame(self)
	local frame = CreateFrame("Frame", "PetBattleMaster", UIParent)
	frame.obj = self
	frame:SetWidth(1)
	frame:SetHeight(1)
	frame:SetMovable(true)
	frame:SetPoint("RIGHT", UIParent, "RIGHT", -250, 0)
	frame:SetClampedToScreen(true)
	frame.SaveWindowPosition = _SaveWindowPosition
	self.frame = frame
	
	self.infoView = petbm.PetInfoView.New(frame)
	self.infoView.frame:SetPoint("CENTER")
	
	frame:Hide()
end

local function Open(self)
	self.frame:Show()
end

local function Close(self)
	self.frame:Hide()
end

--local function CheckOpponent(self)
--	log:Debug("CheckOpponent")
--	local activeEnemy = C_PetBattles.GetActivePet(LE_BATTLE_PET_ENEMY)
--	local petType = C_PetBattles.GetPetType(LE_BATTLE_PET_ENEMY, activeEnemy)
--	log:Debug("activeEnemy [%s] petType [%s]", activeEnemy, petType)
--	if (petType) then
--		self.opponentFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
--		self.opponentFrame:Show()
--		local name, speciesName = C_PetBattles.GetName(LE_BATTLE_PET_ENEMY, activeEnemy)
--		log:Debug("opponent name [%s]", name)
--		local petId, speciesId, owned = C_PetJournal.GetPetInfoByIndex(activeEnemy, C_PetJournal.isWild)
--		log:Debug("petId [%s] speciesId [%s] owned [%s]", petId, speciesId, owned)
--		
--		_UpdateDamageModifiers(self, petType)
--	else
--		self.opponentFrame:Hide()
--	end
--end

local function Update(self)
	log:Debug("Update enter")
	-- set current own pet
	self.infoView:Update()
	log:Debug("Update exit")
end

local function ProfileUpdate(self)
	_RestoreWindow(self)
end

local function New()
	local instance = {}
		
	instance.Open = Open
	instance.Close = Close
	instance.Update = Update
	instance.ProfileUpdate = ProfileUpdate
	
	instance.profile = petbm.PetBattleMaster.db.profile
	
	_InitFrame(instance)
	_RestoreWindow(instance)
	
	return instance
end

petbm.MainView.New = New

