--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}

petbm.PetInfoView = {}

local L = petbm.Locale.GetInstance()
local log = petbm.Debug:new("PetInfoView")

local ALLY_1 = 1
local ALLY_2 = 2
local ALLY_3 = 3
local ENEMY_1 = 4
local ENEMY_2 = 5
local ENEMY_3 = 6

local PORTRAIT_SIZE = 45
local SMALL_PORTRAIT_SIZE = 35
local TAB_SIZE = 60
local SMALL_TAB_SIZE = 50

local BACKDROP_COLOR = {31/255, 28/255, 38/255, 1}
--local BACKDROP_ACTIVE_COLOR = {0xAF/255, 0xE5/255, 0xEA/255, 1} 
local BACKDROP_ACTIVE_COLOR = {1, 1, 1, 1}
local BACKDROP_INACTIVE_COLOR = {31/255, 28/255, 38/255, 0.3}
local BACKDROP_BORDER_COLOR = {1, 1, 1, 1}
local BACKDROP_BORDER_ALLY_COLOR = {0x33/255, 0xCC/255, 0x33/255, 1}
local BACKDROP_BORDER_ENEMY_COLOR = {1, 0x1B / 255, 0x39 / 255, 1}

local TAB_BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
	tile = true, tileSize = 16, edgeSize = 16, 
	insets = { left = 4, right = 4, top = 4, bottom = 4 }
}

local BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
	tile = true, tileSize = 16, edgeSize = 16, 
	insets = { left = 4, right = 4, top = 4, bottom = 4 }
}

local function _UpdateDamageModifiers(self, petType)
	log:Debug("_UpdateDamageModifiers petType [%s]", petType)
	local dealDmgPlus, dealDmgMinus, receiveDmgPlus, receiveDmgMinus = petbm.PetUtil.GetDamageModifiers(petType)
	if (dealDmgPlus) then
		self.dealDamagePlusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[dealDmgPlus])
		petbm.WidgetUtil.AddTooltip(self.dealDamagePlusFrame, L["Deals damage"], L["Deals increased damage to %s"]:format(_G["BATTLE_PET_DAMAGE_NAME_"..dealDmgPlus]))
		self.dealDamagePlusFrame:Show()
	else
		self.dealDamagePlusFrame:Hide()
	end
	if (dealDmgMinus) then
		self.dealDamageMinusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[dealDmgMinus])
		petbm.WidgetUtil.AddTooltip(self.dealDamageMinusFrame, L["Deals damage"], L["Deals reduced damage to %s"]:format(_G["BATTLE_PET_DAMAGE_NAME_"..dealDmgMinus]))
		self.dealDamageMinusFrame:Show()
	else
		self.dealDamageMinusFrame:Hide()
	end
	if (receiveDmgPlus) then
		self.receiveDamagePlusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[receiveDmgPlus])
		petbm.WidgetUtil.AddTooltip(self.receiveDamagePlusFrame, L["Receives damage"], L["Receives increased damage from %s"]:format(_G["BATTLE_PET_DAMAGE_NAME_"..receiveDmgPlus]))
		self.receiveDamagePlusFrame:Show()
	else
		self.receiveDamagePlusFrame:Hide()
	end
	if (receiveDmgMinus) then
		self.receiveDamageMinusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[receiveDmgMinus])
		petbm.WidgetUtil.AddTooltip(self.receiveDamageMinusFrame, L["Receives damage"], L["Receives reduced damage from %s"]:format(_G["BATTLE_PET_DAMAGE_NAME_"..receiveDmgMinus]))
		self.receiveDamageMinusFrame:Show()
	else
		self.receiveDamageMinusFrame:Hide()
	end
end

local function _IsMissing(self, petIndex)
	local rtn
	
	local quality = C_PetBattles.GetBreedQuality(LE_BATTLE_PET_ENEMY, petIndex)
	local speciesId = C_PetBattles.GetPetSpeciesID(LE_BATTLE_PET_ENEMY, petIndex)
	log:Debug("IsMissing quality [%s] speciesId [%s]", quality, speciesId)
	
	-- reset filter
	C_PetJournal.SetSearchFilter("")
	local numPets = C_PetJournal.GetNumPets(false)
	log:Debug("numPets [%s]", numPets)
	for i = 1,numPets do
		local petId, specId, owned = C_PetJournal.GetPetInfoByIndex(i, false)
		if (specId == speciesId) then
			if (rtn == nil) then
				rtn = true
			end
			if (owned) then
				local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petId)
				log:Debug("found pet with rarity [%s]", rarity)
				if (rarity >= quality) then
					rtn = false
					break
				end
			end
		end
	end
	return rtn
end

local function _UpdateTab(self, tab, petOwner, petIndex)
	local tabFrame = self.tabs[tab]
	if (petIndex > C_PetBattles.GetNumPets(petOwner)) then
		tabFrame:Hide()
	else
		local icon = C_PetBattles.GetIcon(petOwner, petIndex)
		local level = C_PetBattles.GetLevel(petOwner, petIndex)
		local petType = C_PetBattles.GetPetType(petOwner, petIndex)
		-- TODO is this correct???
		local quality = C_PetBattles.GetBreedQuality(petOwner, petIndex)
		log:Debug("petOwner [%s] petIndex [%s] quality [%s]", petOwner, petIndex, quality)
--		local petId, speciesId, owned = C_PetJournal.GetPetInfoByIndex(petIndex, C_PetJournal.isWild)
		local color = ITEM_QUALITY_COLORS[1]
		if (quality) then
			color = ITEM_QUALITY_COLORS[quality-1]
		else
			log:Debug("Pet not found")
		end
		tabFrame:SetBackdropBorderColor(color.r, color.g, color.b, 1)
		
		tabFrame.portrait.icon:SetTexture(icon)
		tabFrame.portrait.level:SetText(level)
		tabFrame.portrait.petType:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
		tabFrame:Show()
		
		if (petOwner == LE_BATTLE_PET_ALLY or self.isWild ~= true or not _IsMissing(self, petIndex)) then
			tabFrame.portrait.catch:Hide()
			tabFrame.portrait.catchBack:Hide()
		else
			tabFrame.portrait.catchBack:Show()
			tabFrame.portrait.catch:Show()
		end
	end
end

local function _UpdateAllTabs(self)
	local off = ALLY_2
	for petIdx=1, 3 do
		if (C_PetBattles.GetActivePet(LE_BATTLE_PET_ALLY) == petIdx) then
			_UpdateTab(self, ALLY_1, LE_BATTLE_PET_ALLY, petIdx)
		else
			_UpdateTab(self, off, LE_BATTLE_PET_ALLY, petIdx)
			off = off + 1
		end
	end

	local off = ENEMY_2
	for petIdx=1, 3 do
		if (C_PetBattles.GetActivePet(LE_BATTLE_PET_ENEMY) == petIdx) then
			_UpdateTab(self, ENEMY_1, LE_BATTLE_PET_ENEMY, petIdx)
		else
			_UpdateTab(self, off, LE_BATTLE_PET_ENEMY, petIdx)
			off = off + 1
		end
	end
end

local function _UpdateTabs(self)
	log:Debug("_UpdateTabs active tab [%s]", self.tab)
	
	_UpdateAllTabs(self)
	
	for tabIdx = ALLY_1, ENEMY_3 do
		local tab = self.tabs[tabIdx]
		if (self.tab == tabIdx) then
			-- activate tab
			log:Debug("Activate tabe [%s]", tabIdx)
			tab:SetBackdropColor(unpack(BACKDROP_ACTIVE_COLOR))
		else
			-- deactivate tab
			log:Debug("Deactivate tabe [%s]", tabIdx)
			tab:SetBackdropColor(unpack(BACKDROP_INACTIVE_COLOR))
		end
	end
		
	local petOwner
	local petIndex
	if (self.tab < ENEMY_1) then
		petOwner = LE_BATTLE_PET_ALLY
		petIndex = self.tab
	else
		petOwner = LE_BATTLE_PET_ENEMY
		petIndex = self.tab - 3
	end
	local name = C_PetBattles.GetName(petOwner, petIndex)
	local petType = C_PetBattles.GetPetType(petOwner, petIndex)

	if (name and petType) then
		self.petName:SetText(name)
		_UpdateDamageModifiers(self, petType)
	end
end

local function _SwitchTab(frame)
	local self = frame.obj
	self.tab = frame.tab
	_UpdateTabs(self)
end

local function _InitPetTypeFrame(self, parent, width, height)
	local frame = CreateFrame("Frame", nil, parent)
	frame.obj = self
	frame:SetWidth(width or 33)
	frame:SetHeight(height or 33)
	local texture = frame:CreateTexture(nil, "ARTWORK")
	texture:SetAllPoints(frame)
	texture:SetTexCoord(0.79687500, 0.49218750, 0.50390625, 0.65625000)
	frame.icon = texture
	return frame
end

local function _InitPetAbilityBadgeFrame(self, parent, width, height, outline, positive)
	local frame = _InitPetTypeFrame(self, parent, width, height)
	local texture = frame:CreateTexture(nil, "BORDER")
	texture:SetWidth(width + outline)
	texture:SetHeight(height + outline)
	texture:SetPoint("CENTER")
	if (positive) then
		texture:SetTexture("Interface\\Addons\\PetBattleMaster\\resources\\PositiveBadge")
	else
		texture:SetTexture("Interface\\Addons\\PetBattleMaster\\resources\\NegativeBadge")
	end
	return frame
end

local function _InitModifierFrame(self)
	local frame = CreateFrame("Frame", nil, self.frame)
	frame.obj = self
	frame:SetWidth(128)
	frame:SetHeight(90)
	petbm.MovingView.AttachToChildFrame(frame)
	
	-- attack modifiers
	local MOD_ICON_SIZE = 50
	local texture = frame:CreateTexture(nil, "BACKGROUND")
	texture:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
	texture:SetWidth(MOD_ICON_SIZE)
	texture:SetHeight(MOD_ICON_SIZE)
	texture:SetTexture("Interface\\HELPFRAME\\HelpIcon-ItemRestoration")

	-- defense modifiers
	local texture = frame:CreateTexture(nil, "BACKGROUND")
	texture:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 0)
	texture:SetWidth(MOD_ICON_SIZE)
	texture:SetHeight(MOD_ICON_SIZE)
	texture:SetTexture("Interface\\HELPFRAME\\HelpIcon-AccountSecurity")
	
	local MOD_OFF = 60
	local MOD_DIST = 25
	local dealDamagePlusFrame = _InitPetAbilityBadgeFrame(self, frame, 30, 30, 14, true)
	dealDamagePlusFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", MOD_OFF, -10)
	dealDamagePlusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-Dragon")
	self.dealDamagePlusFrame = dealDamagePlusFrame
	
	local dealDamageMinusFrame = _InitPetAbilityBadgeFrame(self, frame, 30, 30, 14, false)
	dealDamageMinusFrame:SetPoint("LEFT", dealDamagePlusFrame, "RIGHT", MOD_DIST, 0)
	dealDamageMinusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-Critter")
	self.dealDamageMinusFrame = dealDamageMinusFrame
	
	local receiveDamageMinusFrame = _InitPetAbilityBadgeFrame(self, frame, 30, 30, 14, true)
	receiveDamageMinusFrame:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", MOD_OFF, 10)
	receiveDamageMinusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-Dragon")
	self.receiveDamageMinusFrame = receiveDamageMinusFrame
	
	local receiveDamagePlusFrame = _InitPetAbilityBadgeFrame(self, frame, 30, 30, 14, false)
	receiveDamagePlusFrame:SetPoint("LEFT", receiveDamageMinusFrame, "RIGHT", MOD_DIST, 0)
	receiveDamagePlusFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-Critter")
	self.receiveDamagePlusFrame = receiveDamagePlusFrame
	
	return frame
end

--local function _InitCatchFrame(self, parent)
--	local frame = CreateFrame("Button", "petbmCatchFrame", parent, "PetBattleActionButtonTemplate")
--	frame:SetWidth(26)
--	frame:SetHeight(26)
--	frame.Icon:SetTexture("Interface\\Icons\\INV_Pet_PetTrap")
--	return frame
--end

local function _InitPortraitFrame(self, parent, tab, portraitSize)
	local frame = CreateFrame("Button", nil, parent)
	frame:SetWidth(portraitSize)
	frame:SetHeight(portraitSize)
	frame.obj = self
	frame.tab = tab
	frame:SetScript("OnClick", _SwitchTab)
	parent.portrait = frame
	
	-- background
	local texture = frame:CreateTexture(nil, "BORDER")
	texture:SetAllPoints(frame)
	texture:SetTexture("Interface\\PetBattles\\PetJournal")
	texture:SetTexCoord(0.41992188, 0.52343750, 0.02246094, 0.07519531)
	
	-- portrait icon
	local icon = frame:CreateTexture(nil, "BACKGROUND")
	icon:SetWidth(portraitSize - 18)
	icon:SetHeight(portraitSize - 18)
	icon:SetPoint("TOPLEFT", 10, -8)
	if (tab >= ENEMY_1) then
		icon:SetTexCoord(0, 1, 0, 1)
	end
	frame.icon = icon
	
	-- level
	local levelBack = frame:CreateTexture(nil, "OVERLAY", "MainPet-LevelBubble")
	levelBack:SetWidth(20)
	levelBack:SetHeight(20)
	levelBack:ClearAllPoints()
	levelBack:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 3, -3)

	local level = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	level:SetJustifyH("CENTER")
	level:SetPoint("CENTER", levelBack, "CENTER", 0, 0)
	level:SetText("99")
	frame.level = level

	-- type
	local typeBack = frame:CreateTexture(nil, "OVERLAY")
	typeBack:SetWidth(20)
	typeBack:SetHeight(20)
	typeBack:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", -3, -3)
	typeBack:SetTexCoord(0.79687500, 0.49218750, 0.50390625, 0.65625000)
	typeBack:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[1])
	frame.petType = typeBack

	-- catch
	
	local catchBack = frame:CreateTexture(nil, "ARTWORK", "MainPet-LevelBubble")
	catchBack:SetWidth(20)
	catchBack:SetHeight(20)
	catchBack:ClearAllPoints()
	catchBack:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 3, 3)
	frame.catchBack = catchBack
	local catchName = "petbmCatch"..tab
	local catch = frame:CreateTexture(catchName, "OVERLAY")
	catch:SetWidth(15)
	catch:SetHeight(15)
	catch:SetPoint("CENTER", catchBack, "CENTER", 0, 0)
	SetPortraitToTexture(catchName, "Interface\\Icons\\INV_Pet_PetTrap")
	frame.catch = catch
	
	return frame
end

local function _InitTabFrame(self, parent, tab, tabSize, portraitSize)
	local frame = CreateFrame("Button", nil, parent)
	frame:SetWidth(tabSize)
	frame:SetHeight(tabSize)
	frame.obj = self
	frame.tab = tab
	frame:SetScript("OnClick", _SwitchTab)
	
	frame:SetBackdrop(TAB_BACKDROP)
	frame:SetBackdropColor(unpack(BACKDROP_INACTIVE_COLOR))
--	if (tab < ENEMY_1) then
--		frame:SetBackdropBorderColor(unpack(BACKDROP_BORDER_ALLY_COLOR))
--	else
--		frame:SetBackdropBorderColor(unpack(BACKDROP_BORDER_ENEMY_COLOR))
--	end
	
	frame.portrait = _InitPortraitFrame(self, frame, tab, portraitSize)
	frame.portrait:SetPoint("CENTER", frame, "CENTER", 0, 0)

	self.tabs[tab] = frame
		
	return frame
end

local function _InitFrame(self, parent)
	local frame = CreateFrame("Frame", "petbmInfoView", parent)
	frame.obj = self
	frame:SetWidth(160)
	frame:SetHeight(115)
	self.frame = frame
	petbm.MovingView.AttachToChildFrame(frame)
	
	frame:SetBackdrop(BACKDROP)
	frame:SetBackdropColor(unpack(BACKDROP_COLOR))
	frame:SetBackdropBorderColor(unpack(BACKDROP_BORDER_COLOR))
	
	self.tabs = {}
	
	-- ally tabs
	local tab = _InitTabFrame(self, frame, ALLY_1, TAB_SIZE, PORTRAIT_SIZE)
	tab:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 5, -3)

	local tab = _InitTabFrame(self, frame, ALLY_2, SMALL_TAB_SIZE, SMALL_PORTRAIT_SIZE)
	tab:SetPoint("TOPRIGHT", frame, "TOPLEFT", 2, 0)

	local tab = _InitTabFrame(self, frame, ALLY_3, SMALL_TAB_SIZE, SMALL_PORTRAIT_SIZE)
	tab:SetPoint("TOP", self.tabs[ALLY_2], "BOTTOM", 0, 0)
	
	-- enemy tabs
	local tab = _InitTabFrame(self, frame, ENEMY_1, TAB_SIZE, PORTRAIT_SIZE)
	tab:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", -5, -3)

	local tab = _InitTabFrame(self, frame, ENEMY_2, SMALL_TAB_SIZE, SMALL_PORTRAIT_SIZE)
	tab:SetPoint("TOPLEFT", frame, "TOPRIGHT", -2, 0)

	local tab = _InitTabFrame(self, frame, ENEMY_3, SMALL_TAB_SIZE, SMALL_PORTRAIT_SIZE)
	tab:SetPoint("TOP", self.tabs[ENEMY_2], "BOTTOM", 0, 0)
	
	-- current opponent
	local opponentFrame = _InitPetTypeFrame(self, frame, SMALL_TAB_SIZE, SMALL_PORTRAIT_SIZE)
	opponentFrame:SetPoint("CENTER", frame, "CENTER", -22, -4)
	--opponentFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-Dragon")
	self.opponentFrame = opponentFrame
	
	-- possession indicator
--	local possessFrame = _InitCatchFrame(self, frame)
--	possessFrame:SetPoint("CENTER", frame, "CENTER", 22, -4)
--	opponentFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-Dragon")
--	self.possessFrame = possessFrame

	-- name of the selected pet	
	local petName = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	petName:SetJustifyH("LEFT")
	petName:SetWidth(130)
	petName:SetHeight(12)
	petName:SetPoint("TOPLEFT", frame, "TOPLEFT", 8, -10)
	petName:SetText("Pantherjunges")
	self.petName = petName
	
	-- damage modifier frame
	local modifierFrame = _InitModifierFrame(self)
	modifierFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -25)
end

local function Update(self)
	-- Update is called after a pet has changed
	-- TODO check whether ally or enemy has changed
	self.isWild = C_PetBattles.IsWildBattle()
	log:Debug("Update isWild [%s]", self.isWild)
	self.tab = ALLY_1
	_UpdateTabs(self)
end

local function CheckOpponent(self)
	log:Debug("CheckOpponent")
	local activeEnemy = C_PetBattles.GetActivePet(LE_BATTLE_PET_ENEMY)
	local petType = C_PetBattles.GetPetType(LE_BATTLE_PET_ENEMY, activeEnemy)
	log:Debug("activeEnemy [%s] petType [%s]", activeEnemy, petType)
	if (petType) then
		self.opponentFrame.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
		self.opponentFrame:Show()
		local name, speciesName = C_PetBattles.GetName(LE_BATTLE_PET_ENEMY, activeEnemy)
		log:Debug("opponent name [%s]", name)
		local petId, speciesId, owned = C_PetJournal.GetPetInfoByIndex(activeEnemy, C_PetJournal.isWild)
		log:Debug("petId [%s] speciesId [%s] owned [%s]", petId, speciesId, owned)
		
		_UpdateDamageModifiers(self, petType)
	else
		self.opponentFrame:Hide()
	end
end

local function New(parentFrame)
	local instance = {}
		
	instance.Update = Update
	
	_InitFrame(instance, parentFrame)
	
	return instance
end

petbm.PetInfoView.New = New